import { ReactNode, HTMLProps, useCallback, useRef, useState, useEffect } from 'react';
import styled from 'styled-components';
import { motion } from 'framer-motion';
import theme from '../../../../theme';
import IconButton from '../IconButton/IconButton';

export interface TextAreaProps {
  onClearButtonClicked: (text: string) => void;
  disable?: { isDisabled?: boolean; fillColor?: string };
  text?: { color?: string; size?: number };
  border: { color?: string; hoverColor?: string };
  placeholder: { color?: string; text?: string };
  rows: number;
  children?: ReactNode;
}

export default function TextArea(props: TextAreaProps) {
  const { children, onClearButtonClicked, placeholder, ...textAreaProps } = props;
  const [value, setValue] = useState('');

  const handleChange = useCallback(e => {
    const text = e.target.value;
    setValue(text);
  }, []);

  const handleClearButtonClicked = useCallback(() => {
    setValue('');
    onClearButtonClicked && onClearButtonClicked(value);
  }, [onClearButtonClicked]);

  return (
    <Box>
      <TextAreaStyled value={value} onChange={handleChange} placeholder={placeholder.text} placeholderColor={placeholder.color} {...textAreaProps} />
      <IconButtonArea onClick={handleClearButtonClicked}>
        <IconButton icon="x-circle" color={theme.colors.gray300} backgroundColor="transparent" />
      </IconButtonArea>
    </Box>
  );
}

TextArea.defaultProps = {
  rows: 8,
  placeholder: {
    color: theme.colors.gray300,
    text: 'Placeholder',
  },
  text: {
    color: theme.colors.black,
    size: 16,
  },
  border: {
    color: 'rgba(0,0,0,0.1)',
    hoverColor: 'rgba(0,0,0,0.3)',
  },
  disable: {
    isDisabled: false,
    fillColor: theme.colors.gray50,
  },
  onClearButtonClicked: (text: string) => console.log(`X버튼 클릭됨: ${text}`),
};

const IconButtonArea = styled(motion.div)`
  display: none;
  position: absolute;
  top: 12px;
  right: 20px;
  :active {
    display: block;
  }
`;

const TextAreaStyled = styled(motion.textarea)<any>`
  width: 100%;
  padding: 12px 20px;
  border-radius: 12px;
  border: 1px solid ${({ border }) => border.color};
  cursor: pointer;
  background-color: ${({ disable }) => (disable.isDisabled ? disable.fillColor : theme.colors.white)};
  font-family: ${theme.fontFamily.normal};
  font-size: ${({ text }) => text.size}px;
  line-height: 1.5;
  letter-spacing: -0.3;
  color: ${({ text }) => text.color};

  ::placeholder {
    color: ${({ placeholderColor }) => placeholderColor};
    font-size: ${({ text }) => text.size}px;
    line-height: 1.5;
    letter-spacing: -0.3;
    font-family: ${theme.fontFamily.normal};
  }
  :active,
  :focus,
  :hover {
    outline: none;
    border: 1px solid ${({ border }) => border.hoverColor};
  }
  :active,
  :focus {
    & + ${IconButtonArea} {
      display: block;
    }
    padding: 12px 54px 12px 20px;
  }
`;

const Box = styled(motion.div)`
  display: flex;
  position: relative;
`;
